/*
 * Copyright 2016-2018, The Android Open Source Project
 * Copyright (c) 2021 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define LOG_TAG "FP2_CodecServiceRegistrant"

#include <android-base/logging.h>
#include <android-base/properties.h>
#include <media/stagefright/omx/1.0/Omx.h>
#include <media/stagefright/omx/1.0/OmxStore.h>

using namespace android;
using namespace ::android::hardware::media::omx::V1_0;

static constexpr char const *hwDecProperty = "persist.fp2.use_qcom_hw_vdec";
static constexpr char const* hwDecMainXmlName = "media_codecs_8974_hwdec.xml";

// Called by frameworks/av/services/mediacodec/main_codecservice.cpp
extern "C" void RegisterCodecServices() {
    // Default to software video decoders; only use (legacy) hardware decoders if opted-in.
    const bool use_qcom_hw_vdec = base::GetBoolProperty(hwDecProperty, false);
    const char* mainXmlName = use_qcom_hw_vdec
        ? hwDecMainXmlName
        : MediaCodecsXmlParser::defaultMainXmlName;

    LOG(INFO) << "FP2: Using Qualcomm hardware video decoders: "
        << (use_qcom_hw_vdec ? "yes" : "no");

    sp<IOmxStore> omxStore = new implementation::OmxStore(
            "default",                                  // default parameter value
            MediaCodecsXmlParser::defaultSearchDirs,    // default parameter value
            mainXmlName);

    // Everything below comes from main_codecservice.cpp
    if (omxStore == nullptr) {
        LOG(ERROR) << "Cannot create IOmxStore HAL service.";
    } else if (omxStore->registerAsService() != OK) {
        LOG(ERROR) << "Cannot register IOmxStore HAL service.";
    }
    sp<IOmx> omx = new implementation::Omx();
    if (omx == nullptr) {
        LOG(ERROR) << "Cannot create IOmx HAL service.";
    } else if (omx->registerAsService() != OK) {
        LOG(ERROR) << "Cannot register IOmx HAL service.";
    } else {
        LOG(INFO) << "IOmx HAL service created.";
    }
}
